/* Cookies Directive - The rewrite. Now a jQuery plugin
 * Version: 2.0.0
 * Author: Ollie Phillips
 * 3 October 2013
 */

;(function ($) {
    $.cookiesDirective = function (options) {

        // Default Cookies Directive Settings
        var settings = $.extend({
            //Options
            explicitConsent: true,
            target: '_blank',
            position: 'top',
            duration: 10,
            limit: 0,
            message: null,
            cookieScripts: null,
            privacyPolicyUri: 'privacy.html',
            scriptWrapper: function () {
            },
            // Styling
            fontFamily: 'helvetica',
            fontColor: '#FFFFFF',
            fontSize: '13px',
            backgroundColor: '#000000',
            backgroundOpacity: '80',
            linkColor: '#CA0000'
        }, options);

        disclosure(settings);
    };

    // Used to load external javascript files into the DOM
    $.cookiesDirective.loadScript = function (options) {
        var settings = $.extend({
            uri: '',
            appendTo: 'body'
        }, options);

        var elementId = String(settings.appendTo);
        var sA = document.createElement("script");
        sA.src = settings.uri;
        sA.type = "text/javascript";
        sA.onload = sA.onreadystatechange = function () {
            if ((!sA.readyState || sA.readyState == "loaded" || sA.readyState == "complete")) {
                return;
            }
        }
        switch (settings.appendTo) {
            case 'head':
                $('head').append(sA);
                break;
            case 'body':
                $('body').append(sA);
                break;
            default:
                $('#' + elementId).append(sA);
        }
    }

    // Helper scripts
    // Get cookie
    var getTheCookie = function (name) {
        var nameEQ = name + "=";
        var ca = document.cookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') c = c.substring(1, c.length);
            if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length, c.length);
        }
        return null;
    }

    // Set cookie
    var setTheCookie = function (name, value, days) {
        if (days) {
            var date = new Date();
            date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
            var expires = "; expires=" + date.toGMTString();
        }
        else var expires = "";
        document.cookie = name + "=" + value + expires + "; path=/";
    }

    // Detect IE < 9
    var checkIE = function () {
        var version;
        if (navigator.appName == 'Microsoft Internet Explorer') {
            var ua = navigator.userAgent;
            var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
            if (re.exec(ua) != null) {
                version = parseFloat(RegExp.$1);
            }
            if (version <= 8.0) {
                return true;
            } else {
                if (version == 9.0) {
                    if (document.compatMode == "BackCompat") {
                        // IE9 in quirks mode won't run the script properly, set to emulate IE8
                        var mA = document.createElement("meta");
                        mA.content = "IE=EmulateIE8";
                        document.getElementsByTagName('head')[0].appendChild(mA);
                        return true;
                    } else {
                        return false;
                    }
                }
                return false;
            }
        } else {
            return false;
        }
    }

    // Disclosure routines
    var disclosure = function (options) {
        var settings = options;
        settings.css = 'fixed';
        // IE 9 and lower has issues with position:fixed, either out the box or in compatibility mode - fix that
        if (checkIE()) {
            settings.position = 'top';
            settings.css = 'absolute';
        }

        // Any cookie setting scripts to disclose
        var scriptsDisclosure = '';
        if (settings.cookieScripts) {
            var scripts = settings.cookieScripts.split(',');
            var scriptsCount = scripts.length;
            var scriptDisclosureTxt = '';
            if (scriptsCount > 1) {
                for (var t = 0; t < scriptsCount - 1; t++) {
                    scriptDisclosureTxt += scripts[t] + ', ';
                }
                scriptsDisclosure = ' We use ' + scriptDisclosureTxt.substring(0, scriptDisclosureTxt.length - 2) + ' and ' + scripts[scriptsCount - 1] + ' scripts, which all set cookies. ';
            } else {
                scriptsDisclosure = ' We use a ' + scripts[0] + ' script which sets cookies.';
            }
        }

        // Create overlay, vary the disclosure based on explicit/implied consent
        // Set our disclosure/message if one not supplied
        var html = '';
        html += '<div id="epd">';
        html += '<div id="cookiesdirective" style="position:' + settings.css + ';' + settings.position + ':-300px;left:0px;width:100%;'
        html += 'height:auto;background:' + settings.backgroundColor + ';opacity:.' + settings.backgroundOpacity + ';';
        html += '-ms-filter: “alpha(opacity=' + settings.backgroundOpacity + ')”; filter: alpha(opacity=' + settings.backgroundOpacity + ');';
        html += '-khtml-opacity: .' + settings.backgroundOpacity + '; -moz-opacity: .' + settings.backgroundOpacity + ';';
        html += 'color:' + settings.fontColor + ';font-family:' + settings.fontFamily + ';font-size:' + settings.fontSize + ';';
        html += 'text-align:center;z-index:1000;">';
        html += '<div class="cookiesDirectiveContent">';

        if (settings.usePersonalData && settings.userDeniedCookies == 1) {
            html += window.t('Σας υπενθυμίζουμε πως δεν έχετε αποδεχθεί κάποια προσωπικά cookies και ίσως κάποιες υπηρεσίες να μην σας είναι διαθέσιμες');
            var ok_msg = window.t("Θέλω να τα αποδεχθώ τώρα");
            html += '<input class="acceptCookies cookies_action" type="submit" id="undo_deny" value="' + ok_msg + '"/>';
            html += '</div>';
            html += '</div></div>';
            $('body').append(html);

            // Serve the disclosure, and be smarter about branching
            var dp = settings.position.toLowerCase();
            if (dp != 'top' && dp != 'bottom') {
                dp = 'top';
            }
            var opts = new Array();
            if (dp == 'top') {
                opts['in'] = {'top': '0'};
                opts['out'] = {'top': '-300'};
            } else {
                opts['in'] = {'bottom': '0'};
                opts['out'] = {'bottom': '-300'};
            }

            // Start animation
            $('#cookiesdirective').animate(opts['in'], 1000, function () {
                $('#undo_deny').on('click', function () {
                    reactivateCookies();
                });

                // Set a timer to remove the warning after 'settings.duration' seconds
                setTimeout(function () {
                    $('#cookiesdirective').animate({
                        opacity: '0'
                    }, 2000, function () {
                        $('#cookiesdirective').css(dp, '-300px');
                    });
                }, settings.duration * 1000);
            });
        }

        if (!settings.userDeniedCookies && !getCookie('cookiesDirective')) {

            if (!settings.message) {
                settings.message = '<br>' + window.t('Ο ιστότοπος αυτός χρησιμοποιεί cookies. Κάποια από αυτά, είναι πολύ σημαντικά για την λειτουργία του ιστότοπου και έχουν ήδη τεθεί.');
            }
            html += settings.message;
            // Build the rest of the disclosure for implied and explicit consent
            // Explicit consent disclosure
            html += scriptsDisclosure + window.t('Αν αποκλείσετε αυτά τα απολύτως αναγκαία cookies, αυτός ο ιστότοπος δεν θα είναι καθόλου λειτουργικός.');
            if (settings.usePersonalData) html += '<br><br>' + window.t('Για την καλύτερη και πιο ολοκληρωμένη εμπειρία σας, ίσως χρειαστεί να αποθηκεύσουμε σε cookies κάποιες προσωπικές σας πληροφορίες. Αν δεν συμφωνήσετε σε αυτό, πιθανόν κάποιες υπηρεσίες μας να μην σας είναι διαθέσιμες.');
            html += '<br>' + window.t('Περισσότερα σχετικά με τα cookies αυτού του ιστότοπου, μπορείτε να δείτε στην') + ' <a style="color:' + settings.linkColor + ';font-weight:bold;';
            html += 'font-family:' + settings.fontFamily + ';font-size:' + settings.fontSize + ';" href="' + settings.privacyPolicyUri + '" target="' + settings.target + '">' + window.t('πολιτική απορρήτου') + '</a><br>';
            if (settings.usePersonalData) html += '<input class="denyCookies cookies_action" onclick="denyCookies()" type="submit" id="deny_cookies" value="' + window.t("Δεν αποδέχομαι") + '"/>';
            var ok_msg = (settings.usePersonalData) ? window.t("Αποδέχομαι") : window.t("Το κατάλαβα!");
            html += '<input class="acceptCookies cookies_action" type="submit" name="explicitsubmit" id="explicitsubmit" value="' + ok_msg + '"/>';
            html += '</div>';
            html += '</div></div>';
            $('body').append(html);

            // Serve the disclosure, and be smarter about branching
            var dp = settings.position.toLowerCase();
            if (dp != 'top' && dp != 'bottom') {
                dp = 'top';
            }
            var opts = new Array();
            if (dp == 'top') {
                opts['in'] = {'top': '0'};
                opts['out'] = {'top': '-300'};
            } else {
                opts['in'] = {'bottom': '0'};
                opts['out'] = {'bottom': '-300'};
            }

            // Start animation
            $('#cookiesdirective').animate(opts['in'], 1000, function () {
                $('#explicitsubmit').on('click', function () {
                    setTheCookie('cookiesDirective', 1, 365); // do not show initial message again
                    $('#cookiesdirective').animate(opts['out'], 1000, function () {
                        $('#cookiesdirective').remove();
                    });
                });

                // Set a timer to remove the warning after 'settings.duration' seconds
                setTimeout(function () {
                    $('#cookiesdirective').animate({
                        opacity: '0'
                    }, 2000, function () {
                        $('#cookiesdirective').css(dp, '-300px');
                    });
                }, settings.duration * 1000);
            });
        }
    }
})(jQuery);