<?php
defined('CMS') or die("This file cannot run this way!");

class gdpr
{

    private $log_actions = array('log', 'data_report', 'data_delete', 'email_confirm');

    function log($data = array()) {
        global $db, $variables;
        $response = array("success" => "", "errors" => array());
        $ip = getUserIP();
        $ip = ($ip) ? $variables->encrypt($ip) : "Could not resolve the user IP";
        $email = $data['email'];
        $created = $this->logTimestamp();
        $action = $data['action'];
        if (!in_array($action, $this->log_actions)) return false; // filter invalid actions
        $token = ($action == "data_report" || $action == "data_delete") ? md5(uniqid($email, true)) : "";
        $status = ($action == "data_report" || $action == "data_delete") ? "pending" : "";
        $message = filter_var($variables->filterQuotes($data['message']), FILTER_SANITIZE_STRING);
        $form_id = $data['form_id']; // in case of consent
        if (empty($message)) {
            if ($action == "data_report") $message = "User request for personal data report";
            if ($action == "data_delete") $message = "User request for personal data erasure";
        }
        if ($form_id && $action == "log") $message .= "\nUser gave consent for form (ID: " . $form_id . ")";
        if (!validEmail($email)) {
            $response['errors'][] = "Invalid user e-mail";
            return $response;
        }
        if (empty($message)) { // no logs without reason messages
            $response['errors'][] = "Cannot log without log message";
            return $response;
        }
        $email = $variables->encrypt($email); // always keep e-mail encrypted
        if ($action == "data_report" || $action == "data_delete") {
            if ($this->getRequestStatus($email, $action, 'is_pending')) {
                $response['errors'][] = t("Υπάρχει ήδη σε εξέλιξη αυτό το αίτημά σας. Αναμένουμε την επιβεβαίωση της ταυτότητας σας μέσω του link που σας αποστείλαμε στο e-mail σας");
                return $response;
            }
            if ($this->getRequestStatus($email, $action, 'is_in_progress')) {
                $response['errors'][] = t("Υπάρχει ήδη σε εξέλιξη αυτό το αίτημά σας. Μόλις έχουμε νεότερα θα σας ενημερώσουμε");
                return $response;
            }
            if (!$this->isUserExists($email) && $_SESSION['gdpr']['ignoreNonExistingUsers']) {
                $response['errors'][] = t("Το e-mail αυτό δεν βρέθηκε στην βάση δεδομένων αυτού του site");
                return $response;
            }
        }
        if (!$db->query("INSERT INTO `gdpr` VALUES('', '$email', '$status', '$token', '$created', '$action', '$message', '$ip')")) {
            $response['errors'][] = "Error executing log query";
            return $response;
        }
        $response['token'] = $token;
        $response['success'] = "ok";
        $response['last_id'] = $db->lastID();
        return $response;
    }

    function getRequestStatus($email = '', $action = '', $type = '') {
        global $db;
        if ($action != "data_report" && $action != "data_delete") return false;
        if ($type != "is_pending" && $type != "is_in_progress") die('fatal error, no valid type is specified');
        // email must be encrypted
        if ($type == "is_pending") {
            $q = "select `id` from `gdpr` where `email`='$email' and `action`='$action' and `status`='pending' and `token`<>'confirmed'";
        } else {
            $q = "select `id` from `gdpr` where `email`='$email' and `action`='$action' and `status`='pending' and `token`='confirmed'";
        }
        $userID = $db->getRecord($q)->id;
        return (int)$userID > 0;
    }

    function sendConfirmationLink($email = '', $gdpr_action = '', $token = '') {
        global $langID, $protocol, $domain_name, $email_address;
        if ($gdpr_action == "data_report") $action = t("Αναφορά προσωπικών δεδομένων");
        if ($gdpr_action == "data_delete") $action = t("Διαγραφή προσωπικών δεδομένων");
        if (!$action) return false;
        $mail_header = "Return-Path: $domain_name <" . $email_address . ">\n";
        $mail_header .= "From: " . $email_address . " <" . $email_address . ">\n";
        $mail_header .= "Organization: $domain_name\n";
        $mail_header .= "Content-Type: text; charset=utf-8\n";
        $subject = "Your confirmation required about a GDPR action requested on " . $domain_name;
        $msg = t("Απαιτείται επιβεβαίωση για αίτηση GDPR που έγινε σχετικά με το e-mail σας από το") . " " . $domain_name . "\n";
        $msg .= t("Η αίτηση αφορά την ακόλουθη ενέργεια") . ":\n";
        $msg .= $action . "\n\n";
        $msg .= t("Αν κάνατε εσείς την αίτηση παρακαλούμε επιβεβαιώστε την, κάνοντας χρήση του ακόλουθου συνδέσμου (link)") . ":\n\n";
        $msg .= $protocol . "://" . $domain_name . "/modules/odyssey_gdpr/confirm.php?langID=" . $langID . "&token=" . $token . "\n\n";
        return mail($email, $subject, $msg, $mail_header);
    }

    function notifyAdmin($gdpr_action = '', $email = '') {
        global $variables, $domain_name, $email_address;
        $admin_email = (validEmail($_SESSION['gdpr']['notifyEmail'])) ? $_SESSION['gdpr']['notifyEmail'] : $email_address;
        if ($gdpr_action == "data_report") $action = t("Αναφορά προσωπικών δεδομένων");
        if ($gdpr_action == "data_delete") $action = t("Διαγραφή προσωπικών δεδομένων");
        $email = $variables->decrypt($email);
        if (!$action || !validEmail($email)) return false;
        $mail_header = "Return-Path: $domain_name <" . $email_address . ">\n";
        $mail_header .= "From: " . $email_address . " <" . $email_address . ">\n";
        $mail_header .= "Organization: $domain_name\n";
        $mail_header .= "Content-Type: text; charset=utf-8\n";
        $subject = "A GDPR action is requested!";
        $msg = t("Επιβεβαιώθηκε αίτηση χρήστη σύμφωνα με τον κανονισμό GDPR και πρέπει να ενεργήσετε") . "\n";
        $msg .= t("Η αίτηση αφορά την ακόλουθη ενέργεια") . ":\n";
        $msg .= $action . "\n\n";
        $msg .= "User e-mail:" . $email . "\n";
        return mail($admin_email, $subject, $msg, $mail_header);
    }

    function removeLog($id = '') {
        global $db;
        $q = "delete from `gdpr` where `id`='$id'";
        $db->query($q);
    }

    function logTimestamp() {
        global $dates;
        $now = time();
        $day = gmdate('d', $now);
        $month = gmdate('m', $now);
        $year = gmdate('Y', $now);
        $time = getdate($now);
        $timestamp = $dates->getGreekTimestamp($day, $month, $year, $time['hours'], $time['minutes']);
        return ($timestamp) ? $timestamp : time();
    }

    function isUserExists($email = '') {
        global $eshop, $db;
        $cms = is_object($eshop) ? "odyshop" : "odyssey";
        $q = "select `id` from `users` where `email`='$email'";
        $userID = $db->getRecord($q)->id;
        if ((int)$userID) return true;
        if ($cms == "odyshop") { // proceed to search in odyshop orders for unregistered users
            $q = "select `id` from `orders` where `email`='$email' limit 1";
            $orderID = $db->getRecord($q)->id;
            if ((int)$orderID > 0) return true;
        }
        return false;
    }

} // end class
?>